/* 
 * $Id: vscan-oav_core.c,v 1.0 2002/03/08 15:45:24 reniar Exp $	
 *
 * Core Interface for OpenAntiVirus ScannerDaemon			
 *
 * Copyright (C) Rainer Link, SuSE GmbH <link@suse.de>, 2001-2002
 *                            OpenAntiVirus.org <rainer@openantivirus.org>
 *
 * Credits to W. Richard Stevens - RIP
 * 
 * This software is licensed under the GNU General Public License (GPL)
 * See: http://www.gnu.org/copyleft/gpl.html
 *
*/

#include <includes.h>
#include "vscan-oav_core.h"
 

/* read n bytes from descriptor
   from UNP from WRS
*/

ssize_t readn(int fd, void *vptr, size_t n)
{

	size_t nleft;
	ssize_t nread;
	char *ptr;

	ptr = vptr;
	nleft = n;
	while ( nleft > 0 ) {
		if ( (nread = read(fd, ptr, nleft)) < 0 ) {
			if ( errno == EINTR )
				nread = 0;	/* cand call read() again */
			else
				return (-1);
		} else if ( nread == 0 )
			break;			/* EOF */

		nleft -= nread;
		ptr += nread;
	}

	return (n - nleft);			/* return >= 0 */

}



/* write n bytes to descriptor
   from UNP from WRS
*/

ssize_t writen(int fd, const void *vptr, size_t n)
{
	size_t nleft;
	ssize_t nwritten;
	const char *ptr;

	ptr = vptr;
	nleft = n;
	while ( nleft > 0 ) {
		if ( (nwritten = write(fd, ptr, nleft)) <= 0 ) {
			if ( errno == EINTR )
				nwritten = 0;	/* and call write() again */
			else
				return (-1);	/* error */
			
		}
		nleft -= nwritten;
		ptr += nwritten;
	}

	return (n);

}


/* print a message via syslog */
void vscan_oav_print(char *printMessage, ...)
{
        char printMsg[512];

        va_list argptr;
        va_start(argptr, printMessage);
        vsnprintf(printMsg, sizeof(printMsg)-1, printMessage, argptr);
        va_end(argptr);

	syslog(SYSLOG_PRIORITY, "%s", printMsg);
}



/* initialise socket to ScannerDaemon
   returns -1 on error or the socket descriptor */
int vscan_oav_init(void)
{

	int sockfd;
	struct sockaddr_in servaddr;

	/* create socket */
        if (( sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0 ) {
               vscan_oav_print("ERROR: can not create socket!\n");
               return -1; 
        }

	bzero(&servaddr, sizeof(servaddr));
        servaddr.sin_family = AF_INET;
        servaddr.sin_port = htons(OAV_PORT);

	/* hm, inet_pton may not exist on all systems - FIXME ! */
        if ( inet_pton(AF_INET, OAV_IP, &servaddr.sin_addr) <= 0 ) {
                vscan_oav_print("ERROR: inet_pton failed!\n");
                return -1;
	}

	/* connect to socket */
	if ( connect(sockfd, (struct sockaddr *) &servaddr, sizeof(servaddr)) < 0 )
        {
                vscan_oav_print("ERROR: can not connect to ScannerDaemon!\n");
                return -1;
        }


	return sockfd;

}

/*
void vscan_oav_syslog(char *syslogMessage, ...)
{
        char logMsg[512];

        va_list argptr;
        va_start(argptr, syslogMessage);
        vsnprintf(logMsg, sizeof(logMsg)-1, syslogMessage, argptr);
        va_end(argptr);

        syslog(SYSLOG_PRIORITY, "%s\n", logMsg);
}

*/

/*
  If virus is found, logs the filename/virusname into syslog
*/
void vscan_oav_log_virus(char *infected_file, char *result)
{
	char *str;

	/* remove "FOUND: " from the result string to get only the virus name */
	str = result;
	str+= 7;

        vscan_oav_print("Scan result: '%s' infected with virus '%s' - Access denied", infected_file, str);
        
}



/*
  Scans a file (*FILE*, not a directory - keep that in mind) for a virus
  Expects socket descriptor and file name to scan for
  Return -1 on error, 0 if no virus was found, 1 if a virus was found 
*/
int vscan_oav_scanfile(int sockfd, char *scan_file)
{
	int n;
	size_t len;
	char recvline[MAXLINE + 1];
	pstring oavCommand;


        #if VSCAN_OAV_VERBOSE_FILE_LOGGING==1
        vscan_oav_print("Scanning file : '%s'", scan_file);
        #endif

	/* ScannerDaemon expects "SCAN <filename>\n" */
	/* what about if the <filename> itself contains '\n'? */
        fstrcpy(oavCommand, "SCAN ");
        fstrcat(oavCommand, scan_file);
        fstrcat(oavCommand, "\n");

	/* write to socket */
	/* NOTE: what happens if scan_file is very long? */
	len = strlen(oavCommand);
	if ( len < 0 ) {
		/* overflow? */
		vscan_oav_print("ERROR: file name is too long!");
		return -1;
	}	

	n = writen(sockfd, oavCommand, len);
	if ( n <= 0 ) {
		vscan_oav_print("ERROR: can not send file name to ScannerDaemon!");
		return -1;
	}

	/* read from socket */
	n = readn(sockfd, recvline, MAXLINE);
	if ( n < 0 ) {
		vscan_oav_print("ERROR: can not get result from ScannerDaemon!");
		return -1;
	}
	if ( n > 0 )
		recvline[n-1] = 0;	/* null terminate */

	/* Response from ScannerDaemon is either
	   - OK                  file scanned and not infected
	   - FOUND: <virusname>  file scanned and infected
	   - ERROR		 sth went wrong, i.e. file not found
	*/
	if ( strncmp("FOUND", recvline, 5) == 0   ) {
		/* virus found */
		vscan_oav_log_virus(scan_file, recvline);
		return 1;
	} else if ( strncmp("ERROR", recvline, 5) == 0  ) {
		/* ERROR */
		#if VSCAN_OAV_VERBOSE_FILE_LOGGING==1
		vscan_oav_print("file %s not found or not readable", scan_file);
		#endif
	} else if ( strncmp("OK", recvline, 2) == 0 ) {
		/* OK */
		#if VSCAN_OAV_VERBOSE_FILE_LOGGING==1
		vscan_oav_print("file %s is clean", scan_file);
		#endif
        } else {
		/* unknown response */
		vscan_oav_print("unknown response from ScannerDaemon while scanning %s!", scan_file);
	}	

	return 0;
	
}

/*
  close socket
*/
void vscan_oav_end(int sockfd)
{
	/* sockfd == -1 indicates an error while connecting to socket */
	if ( sockfd >= 0 ) {
		close(sockfd);
	}

}
